; Author:      Roy Klein Gebbinck (www.b-k-g.nl)
; Version:     20180101: Improved handling of paths (containing loginnames).
;              20171222: Improved alert message.
;              20171219: Revised.
;              20171214: Minor change to ImportSettings.
;              20171213: First version.
; Created for: https://forum.bricsys.com/discussion/32575/user-profile-manager-settings

; ****************************************************************
; ********************* USE AT YOUR OWN RISK *********************
; ****************************************************************

; BricsCAD V17 or higher is required (new getcfg and setcfg):
(if (> 17 (atoi (getvar '_vernum)))
  (progn
    (princ "\nImportSettings requires BricsCAD V17 or higher ")
    (exit)
  )
)

; Library function:
; Based on code by Luis Esquivel. See: https://www.theswamp.org/index.php?topic=7272.msg90039#msg90039
(defun KGA_String_SplitAll (str sub ignoreCaseP / i j len lst srchStr)
  (if (/= 0 (setq len (strlen sub)))
    (progn
      (if ignoreCaseP
        (progn
          (setq srchStr (strcase str))
          (setq sub (strcase sub))
        )
        (setq srchStr str)
      )
      (setq i 0)
      (while (setq j (vl-string-search sub srchStr i))
        (setq lst (cons (substr str (1+ i) (- j i)) lst))
        (setq i (+ j len))
      )
      (reverse (cons (substr str (1+ i)) lst))
    )
  )
)

; Library function:
(defun KGA_String_SubstAll (str old new ignoreCaseP / lst)
  (setq lst (KGA_String_SplitAll str old ignoreCaseP))
  (apply
    'strcat
    (cons
      (car lst)
      (mapcar '(lambda (a) (strcat new a)) (cdr lst))
    )
  )
)

; 20180101: Values pointing to paths that do not exist are skipped.
; 20180101: Paths containing the loginname stored in the CSV file are translated to
;           the current loginname. All occurences of the old name are translated.
; 20180101: Renamed an argument.
; 20171222: Added "CPROFILE" and "WSCURRENT" warning to the alert message.
; 20171219: Revised.
; 20171219: Added the skipVal argument.
; 20171219: Removed 'not saved' check.
; 20171214: Added 'value is already set' check.
; 20171213
; Fields in settings file:
; 0    1         2         3       4             5             6      7
; Name,Save mode,Save type,Restype,Default value,Current value,Status,Title
;
; Arguments:
;   Fnm:     File name or nil. If a file name is provided the alert message is suppressed.
;   Mode:    "DEFAULT" or "CURRENT".
;   skipVal: Values to skip as string or nil, wild-cards allowed, case-insensitve.
; Examples:
;   (ImportSettings nil "CURRENT" nil)
;   (ImportSettings "D:\\Test\\settings.csv" "CURRENT" "*/*,*\\*") ; Skip paths.
;   (ImportSettings "D:\\Test\\settings.csv" "CURRENT" "*V18*")    ; Skip values containing "V18".
(defun ImportSettings (fnm mode skipVal / noAlertP failLst fnm idx lst nme skipValLst userCsv userCur val)
  (setq noAlertP (and fnm))
  (if
    (and
      (or
        fnm
        (setq fnm (getfiled "Select Settings File" "" "csv" 4))
      )
      (setq lst (cddr (vle-file->list fnm nil))) ; Remove headers and empty line.
      (setq lst (mapcar '(lambda (str) (vle-string-split "," str)) lst))
      (or
        noAlertP
        (=
          1
          (vle-alert
            "ImportSettings"
            (strcat
              "\nThis program will try to import non-dwg related settings from:"
              "\n    " fnm
              "\n"
              (if (/= (strcase (vle-nth5 (assoc "CPROFILE" lst))) (strcase (getvar 'cprofile)))
                (strcat
                  "\nProfile mismatch:"
                  "\n    Profile in settings file:"
                  "\n        " (vle-nth5 (assoc "CPROFILE" lst))
                  "\n    Current profile:"
                  "\n        " (getvar 'cprofile)
                  "\n"
                )
                ""
              )
              (if (/= (strcase (vle-nth5 (assoc "WSCURRENT" lst))) (strcase (getvar 'wscurrent)))
                (strcat
                  "\nWorkspace mismatch:"
                  "\n    Workspace in settings file:"
                  "\n        " (vle-nth5 (assoc "WSCURRENT" lst))
                  "\n    Current workspace"
                  "\n        " (getvar 'wscurrent)
                  "\n"
                )
                ""
              )
              "\nAre you sure you want to continue?"
            )
            305
          )
        )
      )
    )
    (progn
      (setq idx (if (= "DEFAULT" (strcase mode)) 4 5))
      (setq skipVal
        (cond
          ((not skipVal)
            nil
          )
          ((= "" skipVal)
            nil
          )
          (T
            (strcase skipVal)
          )
        )
      )
      (setq userCsv (vle-nth5 (assoc "LOGINNAME" lst))) ; Username stored in CSV.
      (setq userCur (getvar 'loginname))
      (foreach sub lst
        (if
          (and
            (getcfg (setq nme (vle-nth0 sub))) ; Check if the setting is available.
            (/= (setq val (nth idx sub)) "")   ; Skip settings that do not have a (default) value.
            (/= (vle-nth1 sub) "drw")          ; Skip settings that are drawing-related.
            (/= (vle-nth6 sub) "read only")    ; Skip read-only settings.
          )
          (cond
            ((and skipVal (wcmatch (strcase val) skipVal))
              (setq skipValLst (cons val skipValLst))
            )
            ((= (getcfg nme) val) ; Value is already set.
              nil
            )
            ((wcmatch val "*/*,*\\*") ; Val is a path.
              (setq val (KGA_String_SubstAll val userCsv userCur T))
              (if
                (not
                  (and
                    (vl-every 'findfile (vle-string-split ";" val)) ; Splitting required for SRCHPATH.
                    (setcfg nme val)
                  )
                )
                (setq failLst (cons nme failLst))
              )
            )
            (T
              (if (not (setcfg nme val))
                (setq failLst (cons nme failLst))
              )
            )
          )
        )
      )
      (if skipValLst
        (progn
          (princ "\n\n****************************************************** ")
          (princ "\nThe following value(s) were skipped: ")
          (foreach str (reverse skipValLst) (princ (strcat "\n  " str)))
          (princ "\n****************************************************** ")
        )
      )
      (if failLst
        (progn
          (princ "\n\n****************************************************** ")
          (princ "\nThe following setting(s) could not be changed: ")
          (foreach str (reverse failLst) (princ (strcat "\n  " str)))
          (princ "\n****************************************************** ")
        )
      )
      (princ "\n\nDone ")
    )
  )
  (princ)
)

; 20171219: The arguments of ImportSettings have changed.
; 20171213
(defun c:ImportSettingsDefaultValues ()
  (ImportSettings nil "DEFAULT" nil)
)

; 20171219: The arguments of ImportSettings have changed.
; 20171213
(defun c:ImportSettingsUserValues ()
  (ImportSettings nil "CURRENT" nil)
)

(princ "\nUse ImportSettingsUserValues or ImportSettingsDefaultValues ")
(princ)
